# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .inbound_sip_trunk_config_request_model import InboundSipTrunkConfigRequestModel
from .outbound_sip_trunk_config_request_model import OutboundSipTrunkConfigRequestModel


class CreateSipTrunkPhoneNumberRequest(UncheckedBaseModel):
    phone_number: str = pydantic.Field()
    """
    Phone number
    """

    label: str = pydantic.Field()
    """
    Label for the phone number
    """

    supports_inbound: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This field is deprecated and will be removed in the future. Whether this phone number supports inbound calls
    """

    supports_outbound: typing.Optional[bool] = pydantic.Field(default=None)
    """
    This field is deprecated and will be removed in the future. Whether this phone number supports outbound calls
    """

    inbound_trunk_config: typing.Optional[InboundSipTrunkConfigRequestModel] = None
    outbound_trunk_config: typing.Optional[OutboundSipTrunkConfigRequestModel] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .allowed_output_formats import AllowedOutputFormats
from .music_only_output_formats import MusicOnlyOutputFormats

MusicAllowedOutputFormats = typing.Union[AllowedOutputFormats, MusicOnlyOutputFormats]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.analysis_scope import AnalysisScope
from ....types.get_conversation_response_model import GetConversationResponseModel
from .raw_client import AsyncRawAnalysisClient, RawAnalysisClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AnalysisClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAnalysisClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAnalysisClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAnalysisClient
        """
        return self._raw_client

    def run(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConversationResponseModel:
        """
        Run the analysis for a conversation using the agent's current evaluation criteria and data collection settings.

        Parameters
        ----------
        conversation_id : str
            ID of the conversation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.analysis.run(
            conversation_id="conversation_id",
        )
        """
        _response = self._raw_client.run(conversation_id, request_options=request_options)
        return _response.data

    def run_evaluation(
        self,
        conversation_id: str,
        *,
        evaluation_id: str,
        scope: typing.Optional[AnalysisScope] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationResponseModel:
        """
        Rerun a specific evaluation for a conversation.

        Parameters
        ----------
        conversation_id : str
            ID of the conversation

        evaluation_id : str
            ID of the single evaluation criterion to rerun.

        scope : typing.Optional[AnalysisScope]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.analysis.run_evaluation(
            conversation_id="conversation_id",
            evaluation_id="evaluation_id",
        )
        """
        _response = self._raw_client.run_evaluation(
            conversation_id, evaluation_id=evaluation_id, scope=scope, request_options=request_options
        )
        return _response.data


class AsyncAnalysisClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAnalysisClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAnalysisClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAnalysisClient
        """
        return self._raw_client

    async def run(
        self, conversation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetConversationResponseModel:
        """
        Run the analysis for a conversation using the agent's current evaluation criteria and data collection settings.

        Parameters
        ----------
        conversation_id : str
            ID of the conversation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.analysis.run(
                conversation_id="conversation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.run(conversation_id, request_options=request_options)
        return _response.data

    async def run_evaluation(
        self,
        conversation_id: str,
        *,
        evaluation_id: str,
        scope: typing.Optional[AnalysisScope] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationResponseModel:
        """
        Rerun a specific evaluation for a conversation.

        Parameters
        ----------
        conversation_id : str
            ID of the conversation

        evaluation_id : str
            ID of the single evaluation criterion to rerun.

        scope : typing.Optional[AnalysisScope]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.analysis.run_evaluation(
                conversation_id="conversation_id",
                evaluation_id="evaluation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.run_evaluation(
            conversation_id, evaluation_id=evaluation_id, scope=scope, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import inspect
import typing
import uuid

import pydantic
import typing_extensions
from .pydantic_utilities import (
    IS_PYDANTIC_V2,
    ModelField,
    UniversalBaseModel,
    get_args,
    get_origin,
    is_literal_type,
    is_union,
    parse_date,
    parse_datetime,
    parse_obj_as,
)
from .serialization import convert_and_respect_annotation_metadata, get_field_to_alias_mapping
from pydantic_core import PydanticUndefined


class UnionMetadata:
    discriminant: str

    def __init__(self, *, discriminant: str) -> None:
        self.discriminant = discriminant


Model = typing.TypeVar("Model", bound=pydantic.BaseModel)


class UncheckedBaseModel(UniversalBaseModel):
    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            extra = pydantic.Extra.allow

    if IS_PYDANTIC_V2:

        @classmethod
        def model_validate(
            cls: typing.Type["Model"],
            obj: typing.Any,
            *args: typing.Any,
            **kwargs: typing.Any,
        ) -> "Model":
            """
            Ensure that when using Pydantic v2's `model_validate` entrypoint we still
            respect our FieldMetadata-based aliasing.
            """
            dealiased_obj = convert_and_respect_annotation_metadata(
                object_=obj,
                annotation=cls,
                direction="read",
            )
            return super().model_validate(dealiased_obj, *args, **kwargs)  # type: ignore[misc]

    @classmethod
    def model_construct(
        cls: typing.Type["Model"],
        _fields_set: typing.Optional[typing.Set[str]] = None,
        **values: typing.Any,
    ) -> "Model":
        # Fallback construct function to the specified override below.
        return cls.construct(_fields_set=_fields_set, **values)

    # Allow construct to not validate model
    # Implementation taken from: https://github.com/pydantic/pydantic/issues/1168#issuecomment-817742836
    @classmethod
    def construct(
        cls: typing.Type["Model"],
        _fields_set: typing.Optional[typing.Set[str]] = None,
        **values: typing.Any,
    ) -> "Model":
        m = cls.__new__(cls)
        fields_values = {}

        if _fields_set is None:
            _fields_set = set(values.keys())

        fields = _get_model_fields(cls)
        populate_by_name = _get_is_populate_by_name(cls)
        field_aliases = get_field_to_alias_mapping(cls)

        for name, field in fields.items():
            # Key here is only used to pull data from the values dict
            # you should always use the NAME of the field to for field_values, etc.
            # because that's how the object is constructed from a pydantic perspective
            key = field.alias
            if (key is None or field.alias == name) and name in field_aliases:
                key = field_aliases[name]

            if key is None or (key not in values and populate_by_name):  # Added this to allow population by field name
                key = name

            if key in values:
                if IS_PYDANTIC_V2:
                    type_ = field.annotation  # type: ignore # Pydantic v2
                else:
                    type_ = typing.cast(typing.Type, field.outer_type_)  # type: ignore # Pydantic < v1.10.15

                fields_values[name] = (
                    construct_type(object_=values[key], type_=type_) if type_ is not None else values[key]
                )
                _fields_set.add(name)
            else:
                default = _get_field_default(field)
                fields_values[name] = default

                # If the default values are non-null act like they've been set
                # This effectively allows exclude_unset to work like exclude_none where
                # the latter passes through intentionally set none values.
                if default != None and default != PydanticUndefined:
                    _fields_set.add(name)

        # Add extras back in
        extras = {}
        pydantic_alias_fields = [field.alias for field in fields.values()]
        internal_alias_fields = list(field_aliases.values())
        for key, value in values.items():
            # If the key is not a field by name, nor an alias to a field, then it's extra
            if (key not in pydantic_alias_fields and key not in internal_alias_fields) and key not in fields:
                if IS_PYDANTIC_V2:
                    extras[key] = value
                else:
                    _fields_set.add(key)
                    fields_values[key] = value

        object.__setattr__(m, "__dict__", fields_values)

        if IS_PYDANTIC_V2:
            object.__setattr__(m, "__pydantic_private__", None)
            object.__setattr__(m, "__pydantic_extra__", extras)
            object.__setattr__(m, "__pydantic_fields_set__", _fields_set)
        else:
            object.__setattr__(m, "__fields_set__", _fields_set)
            m._init_private_attributes()  # type: ignore # Pydantic v1
        return m


def _validate_collection_items_compatible(collection: typing.Any, target_type: typing.Type[typing.Any]) -> bool:
    """
    Validate that all items in a collection are compatible with the target type.

    Args:
        collection: The collection to validate (list, set, or dict values)
        target_type: The target type to validate against

    Returns:
        True if all items are compatible, False otherwise
    """
    if inspect.isclass(target_type) and issubclass(target_type, pydantic.BaseModel):
        for item in collection:
            try:
                # Try to validate the item against the target type
                if isinstance(item, dict):
                    parse_obj_as(target_type, item)
                else:
                    # If it's not a dict, it might already be the right type
                    if not isinstance(item, target_type):
                        return False
            except Exception:
                return False
    return True


def _convert_undiscriminated_union_type(union_type: typing.Type[typing.Any], object_: typing.Any) -> typing.Any:
    inner_types = get_args(union_type)
    if typing.Any in inner_types:
        return object_

    for inner_type in inner_types:
        # Handle lists of objects that need parsing
        if get_origin(inner_type) is list and isinstance(object_, list):
            list_inner_type = get_args(inner_type)[0]
            try:
                if inspect.isclass(list_inner_type) and issubclass(list_inner_type, pydantic.BaseModel):
                    # Validate that all items in the list are compatible with the target type
                    if _validate_collection_items_compatible(object_, list_inner_type):
                        parsed_list = [parse_obj_as(object_=item, type_=list_inner_type) for item in object_]
                        return parsed_list
            except Exception:
                pass

        try:
            if inspect.isclass(inner_type) and issubclass(inner_type, pydantic.BaseModel):
                # Attempt a validated parse until one works
                return parse_obj_as(inner_type, object_)
        except Exception:
            continue

    # If none of the types work, try matching literal fields first, then fall back
    # First pass: try types where all literal fields match the object's values
    for inner_type in inner_types:
        if inspect.isclass(inner_type) and issubclass(inner_type, pydantic.BaseModel):
            fields = _get_model_fields(inner_type)
            literal_fields_match = True

            for field_name, field in fields.items():
                # Check if this field has a Literal type
                if IS_PYDANTIC_V2:
                    field_type = field.annotation  # type: ignore # Pydantic v2
                else:
                    field_type = field.outer_type_  # type: ignore # Pydantic v1

                if is_literal_type(field_type):  # type: ignore[arg-type]
                    field_default = _get_field_default(field)
                    name_or_alias = get_field_to_alias_mapping(inner_type).get(field_name, field_name)
                    # Get the value from the object
                    if isinstance(object_, dict):
                        object_value = object_.get(name_or_alias)
                    else:
                        object_value = getattr(object_, name_or_alias, None)

                    # If the literal field value doesn't match, this type is not a match
                    if object_value is not None and field_default != object_value:
                        literal_fields_match = False
                        break

            # If all literal fields match, try to construct this type
            if literal_fields_match:
                try:
                    return construct_type(object_=object_, type_=inner_type)
                except Exception:
                    continue

    # Second pass: if no literal matches, just return the first successful cast
    for inner_type in inner_types:
        try:
            return construct_type(object_=object_, type_=inner_type)
        except Exception:
            continue


def _convert_union_type(type_: typing.Type[typing.Any], object_: typing.Any) -> typing.Any:
    base_type = get_origin(type_) or type_
    union_type = type_
    if base_type == typing_extensions.Annotated:  # type: ignore[comparison-overlap]
        union_type = get_args(type_)[0]
        annotated_metadata = get_args(type_)[1:]
        for metadata in annotated_metadata:
            if isinstance(metadata, UnionMetadata):
                try:
                    # Cast to the correct type, based on the discriminant
                    for inner_type in get_args(union_type):
                        try:
                            objects_discriminant = getattr(object_, metadata.discriminant)
                        except:
                            objects_discriminant = object_[metadata.discriminant]
                        if inner_type.__fields__[metadata.discriminant].default == objects_discriminant:
                            return construct_type(object_=object_, type_=inner_type)
                except Exception:
                    # Allow to fall through to our regular union handling
                    pass
    return _convert_undiscriminated_union_type(union_type, object_)


def construct_type(*, type_: typing.Type[typing.Any], object_: typing.Any) -> typing.Any:
    """
    Here we are essentially creating the same `construct` method in spirit as the above, but for all types, not just
    Pydantic models.
    The idea is to essentially attempt to coerce object_ to type_ (recursively)
    """
    # Short circuit when dealing with optionals, don't try to coerces None to a type
    if object_ is None:
        return None

    base_type = get_origin(type_) or type_
    is_annotated = base_type == typing_extensions.Annotated  # type: ignore[comparison-overlap]
    maybe_annotation_members = get_args(type_)
    is_annotated_union = is_annotated and is_union(get_origin(maybe_annotation_members[0]))

    if base_type == typing.Any:  # type: ignore[comparison-overlap]
        return object_

    if base_type == dict:
        if not isinstance(object_, typing.Mapping):
            return object_

        key_type, items_type = get_args(type_)
        d = {
            construct_type(object_=key, type_=key_type): construct_type(object_=item, type_=items_type)
            for key, item in object_.items()
        }
        return d

    if base_type == list:
        if not isinstance(object_, list):
            return object_

        inner_type = get_args(type_)[0]
        return [construct_type(object_=entry, type_=inner_type) for entry in object_]

    if base_type == set:
        if not isinstance(object_, set) and not isinstance(object_, list):
            return object_

        inner_type = get_args(type_)[0]
        return {construct_type(object_=entry, type_=inner_type) for entry in object_}

    if is_union(base_type) or is_annotated_union:
        return _convert_union_type(type_, object_)

    # Cannot do an `issubclass` with a literal type, let's also just confirm we have a class before this call
    if (
        object_ is not None
        and not is_literal_type(type_)
        and (
            (inspect.isclass(base_type) and issubclass(base_type, pydantic.BaseModel))
            or (
                is_annotated
                and inspect.isclass(maybe_annotation_members[0])
                and issubclass(maybe_annotation_members[0], pydantic.BaseModel)
            )
        )
    ):
        if IS_PYDANTIC_V2:
            return type_.model_construct(**object_)
        else:
            return type_.construct(**object_)

    if base_type == dt.datetime:
        try:
            return parse_datetime(object_)
        except Exception:
            return object_

    if base_type == dt.date:
        try:
            return parse_date(object_)
        except Exception:
            return object_

    if base_type == uuid.UUID:
        try:
            return uuid.UUID(object_)
        except Exception:
            return object_

    if base_type == int:
        try:
            return int(object_)
        except Exception:
            return object_

    if base_type == bool:
        try:
            if isinstance(object_, str):
                stringified_object = object_.lower()
                return stringified_object == "true" or stringified_object == "1"

            return bool(object_)
        except Exception:
            return object_

    return object_


def _get_is_populate_by_name(model: typing.Type["Model"]) -> bool:
    if IS_PYDANTIC_V2:
        return model.model_config.get("populate_by_name", False)  # type: ignore # Pydantic v2
    return model.__config__.allow_population_by_field_name  # type: ignore # Pydantic v1


PydanticField = typing.Union[ModelField, pydantic.fields.FieldInfo]


# Pydantic V1 swapped the typing of __fields__'s values from ModelField to FieldInfo
# And so we try to handle both V1 cases, as well as V2 (FieldInfo from model.model_fields)
def _get_model_fields(
    model: typing.Type["Model"],
) -> typing.Mapping[str, PydanticField]:
    if IS_PYDANTIC_V2:
        return model.model_fields  # type: ignore # Pydantic v2
    else:
        return model.__fields__  # type: ignore # Pydantic v1


def _get_field_default(field: PydanticField) -> typing.Any:
    try:
        value = field.get_default()  # type: ignore # Pydantic < v1.10.15
    except:
        value = field.default
    if IS_PYDANTIC_V2:
        from pydantic_core import PydanticUndefined

        if value == PydanticUndefined:
            return None
        return value
    return value



# This file was auto-generated by Fern from our API Definition.

import typing

VoiceDesignRequestModelModelId = typing.Union[typing.Literal["eleven_multilingual_ttv_v2", "eleven_ttv_v3"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

BatchCallStatus = typing.Union[typing.Literal["pending", "in_progress", "completed", "failed", "cancelled"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.language_added_response import LanguageAddedResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawLanguageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add(
        self,
        dubbing_id: str,
        *,
        language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[LanguageAddedResponse]:
        """
        Adds the given ElevenLab Turbo V2/V2.5 language code to the resource. Does not automatically generate transcripts/translations/audio.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language : typing.Optional[str]
            The Target language.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[LanguageAddedResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/language",
            method="POST",
            json={
                "language": language,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LanguageAddedResponse,
                    construct_type(
                        type_=LanguageAddedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLanguageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add(
        self,
        dubbing_id: str,
        *,
        language: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[LanguageAddedResponse]:
        """
        Adds the given ElevenLab Turbo V2/V2.5 language code to the resource. Does not automatically generate transcripts/translations/audio.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language : typing.Optional[str]
            The Target language.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[LanguageAddedResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/language",
            method="POST",
            json={
                "language": language,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    LanguageAddedResponse,
                    construct_type(
                        type_=LanguageAddedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_platform_settings_request_model import AgentPlatformSettingsRequestModel
from .agent_workflow_request_model import AgentWorkflowRequestModel
from .conversational_config import ConversationalConfig


class AdhocAgentConfigOverrideForTestRequestModel(UncheckedBaseModel):
    conversation_config: ConversationalConfig
    platform_settings: AgentPlatformSettingsRequestModel
    workflow: typing.Optional[AgentWorkflowRequestModel] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(AdhocAgentConfigOverrideForTestRequestModel)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.list_auth_connections_response import ListAuthConnectionsResponse
from .types.auth_connections_create_request_body import AuthConnectionsCreateRequestBody
from .types.auth_connections_create_response import AuthConnectionsCreateResponse
from .types.auth_connections_update_request_body import AuthConnectionsUpdateRequestBody
from .types.auth_connections_update_response import AuthConnectionsUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAuthConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListAuthConnectionsResponse]:
        """
        Get all auth connections for the workspace

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListAuthConnectionsResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/workspace/auth-connections",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListAuthConnectionsResponse,
                    construct_type(
                        type_=ListAuthConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: AuthConnectionsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AuthConnectionsCreateResponse]:
        """
        Create a new OAuth2 auth connection for the workspace

        Parameters
        ----------
        request : AuthConnectionsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthConnectionsCreateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/workspace/auth-connections",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AuthConnectionsCreateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthConnectionsCreateResponse,
                    construct_type(
                        type_=AuthConnectionsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, auth_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Any]:
        """
        Delete an auth connection

        Parameters
        ----------
        auth_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/workspace/auth-connections/{jsonable_encoder(auth_connection_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        auth_connection_id: str,
        *,
        request: AuthConnectionsUpdateRequestBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AuthConnectionsUpdateResponse]:
        """
        Update an auth connection

        Parameters
        ----------
        auth_connection_id : str

        request : AuthConnectionsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuthConnectionsUpdateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/workspace/auth-connections/{jsonable_encoder(auth_connection_id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AuthConnectionsUpdateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthConnectionsUpdateResponse,
                    construct_type(
                        type_=AuthConnectionsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAuthConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListAuthConnectionsResponse]:
        """
        Get all auth connections for the workspace

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListAuthConnectionsResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/workspace/auth-connections",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListAuthConnectionsResponse,
                    construct_type(
                        type_=ListAuthConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: AuthConnectionsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AuthConnectionsCreateResponse]:
        """
        Create a new OAuth2 auth connection for the workspace

        Parameters
        ----------
        request : AuthConnectionsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthConnectionsCreateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/workspace/auth-connections",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AuthConnectionsCreateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthConnectionsCreateResponse,
                    construct_type(
                        type_=AuthConnectionsCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, auth_connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Delete an auth connection

        Parameters
        ----------
        auth_connection_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/workspace/auth-connections/{jsonable_encoder(auth_connection_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        auth_connection_id: str,
        *,
        request: AuthConnectionsUpdateRequestBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AuthConnectionsUpdateResponse]:
        """
        Update an auth connection

        Parameters
        ----------
        auth_connection_id : str

        request : AuthConnectionsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuthConnectionsUpdateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/workspace/auth-connections/{jsonable_encoder(auth_connection_id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AuthConnectionsUpdateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuthConnectionsUpdateResponse,
                    construct_type(
                        type_=AuthConnectionsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

EmbeddingModelEnum = typing.Union[
    typing.Literal["e5_mistral_7b_instruct", "multilingual_e5_large_instruct"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.project_snapshot_extended_response_model import ProjectSnapshotExtendedResponseModel
from ....types.project_snapshots_response import ProjectSnapshotsResponse
from .raw_client import AsyncRawSnapshotsClient, RawSnapshotsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SnapshotsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSnapshotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSnapshotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSnapshotsClient
        """
        return self._raw_client

    def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectSnapshotsResponse:
        """
        Retrieves a list of snapshots for a Studio project.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectSnapshotsResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.snapshots.list(
            project_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.list(project_id, request_options=request_options)
        return _response.data

    def get(
        self, project_id: str, project_snapshot_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectSnapshotExtendedResponseModel:
        """
        Returns the project snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectSnapshotExtendedResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.snapshots.get(
            project_id="21m00Tcm4TlvDq8ikWAM",
            project_snapshot_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get(project_id, project_snapshot_id, request_options=request_options)
        return _response.data

    def stream(
        self,
        project_id: str,
        project_snapshot_id: str,
        *,
        convert_to_mpeg: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Stream the audio from a Studio project snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        convert_to_mpeg : typing.Optional[bool]
            Whether to convert the audio to mpeg format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.snapshots.stream(
            project_id="project_id",
            project_snapshot_id="project_snapshot_id",
        )
        """
        with self._raw_client.stream(
            project_id, project_snapshot_id, convert_to_mpeg=convert_to_mpeg, request_options=request_options
        ) as r:
            yield from r.data

    def stream_archive(
        self, project_id: str, project_snapshot_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Returns a compressed archive of the Studio project's audio.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Streaming archive data

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.snapshots.stream_archive(
            project_id="project_id",
            project_snapshot_id="project_snapshot_id",
        )
        """
        with self._raw_client.stream_archive(project_id, project_snapshot_id, request_options=request_options) as r:
            yield from r.data


class AsyncSnapshotsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSnapshotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSnapshotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSnapshotsClient
        """
        return self._raw_client

    async def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectSnapshotsResponse:
        """
        Retrieves a list of snapshots for a Studio project.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectSnapshotsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.snapshots.list(
                project_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project_id, request_options=request_options)
        return _response.data

    async def get(
        self, project_id: str, project_snapshot_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectSnapshotExtendedResponseModel:
        """
        Returns the project snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the Studio project.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectSnapshotExtendedResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.snapshots.get(
                project_id="21m00Tcm4TlvDq8ikWAM",
                project_snapshot_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(project_id, project_snapshot_id, request_options=request_options)
        return _response.data

    async def stream(
        self,
        project_id: str,
        project_snapshot_id: str,
        *,
        convert_to_mpeg: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Stream the audio from a Studio project snapshot.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        convert_to_mpeg : typing.Optional[bool]
            Whether to convert the audio to mpeg format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.snapshots.stream(
                project_id="project_id",
                project_snapshot_id="project_snapshot_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.stream(
            project_id, project_snapshot_id, convert_to_mpeg=convert_to_mpeg, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def stream_archive(
        self, project_id: str, project_snapshot_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Returns a compressed archive of the Studio project's audio.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        project_snapshot_id : str
            The ID of the Studio project snapshot.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Streaming archive data

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.snapshots.stream_archive(
                project_id="project_id",
                project_snapshot_id="project_snapshot_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.stream_archive(
            project_id, project_snapshot_id, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel


class WorkflowToolResponseModelInput(UncheckedBaseModel):
    """
    A common model for workflow tool responses.
    """

    steps: typing.Optional[typing.List["WorkflowToolResponseModelInputStepsItem"]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .workflow_tool_response_model_input_steps_item import WorkflowToolResponseModelInputStepsItem  # noqa: E402, I001

update_forward_refs(WorkflowToolResponseModelInput)



# This file was auto-generated by Fern from our API Definition.

import typing

BodyEditServiceAccountApiKeyV1ServiceAccountsServiceAccountUserIdApiKeysApiKeyIdPatchAllowedIps = typing.Union[
    typing.List[str], typing.Literal["clear"], typing.Literal["no_update"]
]



"""SpeechEngineResource — client-facing handle for a speech engine instance."""

import base64
import hashlib
import hmac
import json
import logging
import time
import typing

from .server import SpeechEngineServer
from .session import SpeechEngineSession
from .types import WebSocketLike
from ..types.speech_engine_response import SpeechEngineResponse

logger = logging.getLogger("elevenlabs.speech_engine")

_ISSUER = "https://api.elevenlabs.io/convai/speech-engine"
_SUBJECT = "convai_speech_engine_upstream"
_LEEWAY_SECONDS = 60


def _base64url_decode(data: str) -> bytes:
    padded = data.replace("-", "+").replace("_", "/")
    remainder = len(padded) % 4
    if remainder:
        padded += "=" * (4 - remainder)
    return base64.b64decode(padded)


def verify_speech_engine_jwt(value: str, api_key: str) -> typing.Dict[str, typing.Any]:
    """Verify an HS256 JWT from the ElevenLabs Speech Engine API.

    The HMAC secret is the SHA-256 hash of the API key.  Returns the
    decoded payload on success, raises :class:`ValueError` on failure.
    """
    token = value.strip()
    if token.lower().startswith("bearer "):
        token = token[7:].strip()

    parts = token.split(".")
    if len(parts) != 3:
        raise ValueError("Invalid JWT: expected 3 parts")

    header_b64, payload_b64, signature_b64 = parts

    try:
        payload = json.loads(_base64url_decode(payload_b64))
    except Exception:
        raise ValueError("Invalid JWT: failed to decode payload")

    trimmed_key = api_key.strip()
    secret = hashlib.sha256(trimmed_key.encode("utf-8")).digest()

    expected_sig = hmac.new(
        secret, f"{header_b64}.{payload_b64}".encode(), hashlib.sha256
    ).digest()
    actual_sig = _base64url_decode(signature_b64)

    if not hmac.compare_digest(expected_sig, actual_sig):
        raise ValueError(
            "Invalid JWT: signature mismatch — make sure the API key used "
            "by your Speech Engine server matches the one used to create "
            "the engine."
        )

    if payload.get("iss") != _ISSUER:
        raise ValueError(
            f'Invalid JWT: expected issuer "{_ISSUER}", got "{payload.get("iss")}"'
        )
    if payload.get("sub") != _SUBJECT:
        raise ValueError(
            f'Invalid JWT: expected subject "{_SUBJECT}", got "{payload.get("sub")}"'
        )

    now = int(time.time())

    exp = payload.get("exp")
    if not isinstance(exp, (int, float)):
        raise ValueError("Invalid JWT: missing exp claim")
    iat = payload.get("iat")
    if not isinstance(iat, (int, float)):
        raise ValueError("Invalid JWT: missing iat claim")
    if exp + _LEEWAY_SECONDS < now:
        raise ValueError("Invalid JWT: token has expired")
    if iat - _LEEWAY_SECONDS > now:
        raise ValueError("Invalid JWT: iat is in the future")

    return payload


class SpeechEngineResource:
    """Represents a speech engine instance.

    Returned by ``await client.speech_engine.get("seng_123")``.

    Use :meth:`serve` to start a standalone WebSocket server, or
    :meth:`create_session` to wrap an existing WebSocket for custom
    server integration (FastAPI, Starlette, etc.).

    Example::

        engine = await elevenlabs.speech_engine.get("seng_123")

        async def on_transcript(transcript, session):
            stream = await openai.responses.create(...)
            await session.send_response(stream)

        await engine.serve(
            port=3001,
            debug=True,
            on_transcript=on_transcript,
        )
    """

    def __init__(
        self,
        engine_id: str,
        client_wrapper: typing.Any = None,
        response: typing.Optional[SpeechEngineResponse] = None,
    ) -> None:
        self.engine_id = engine_id
        self._client_wrapper = client_wrapper

        # Full API response, accessible as engine.config.asr, engine.config.overrides, etc.
        # None when the resource is constructed directly without an API call.
        self.config: typing.Optional[SpeechEngineResponse] = response

    def _get_api_key(self) -> typing.Optional[str]:
        if self._client_wrapper is None:
            return None
        headers = self._client_wrapper.get_headers()
        return headers.get("xi-api-key")

    def verify_request(
        self, headers: typing.Dict[str, typing.Any]
    ) -> bool:
        """Verify that an incoming request is from the ElevenLabs API.

        Checks the ``X-Elevenlabs-Speech-Engine-Authorization`` header
        for a valid JWT signed with the SHA-256 hash of the API key.

        Only needed when managing the WebSocket upgrade yourself.
        When using :meth:`serve`, verification is handled automatically.
        """
        api_key = self._get_api_key()
        if not api_key:
            return False
        raw = headers.get("x-elevenlabs-speech-engine-authorization")
        if isinstance(raw, list):
            raw = raw[0] if raw else None
        if not raw:
            return False
        try:
            verify_speech_engine_jwt(raw, api_key)
            return True
        except ValueError:
            return False

    async def serve(
        self,
        *,
        port: int = 3001,
        path: typing.Optional[str] = None,
        debug: bool = False,
        **handlers: typing.Any,
    ) -> None:
        """Start a standalone WebSocket server.  Blocks until stopped."""
        api_key = self._get_api_key()
        server = SpeechEngineServer(
            port=port, path=path, debug=debug, api_key=api_key, **handlers
        )
        await server.serve()

    def create_session(
        self,
        ws: WebSocketLike,
        *,
        debug: bool = False,
    ) -> SpeechEngineSession:
        """Wrap *ws* in a :class:`SpeechEngineSession`.

        Use this for custom server integration (e.g. FastAPI, Starlette).
        Wire handlers via :meth:`~SpeechEngineSession.on` then ``await
        session.run()``.
        """
        return SpeechEngineSession(ws, debug=debug)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .threshold_guardrail import ThresholdGuardrail


class ModerationConfig(UncheckedBaseModel):
    sexual: typing.Optional[ThresholdGuardrail] = None
    violence: typing.Optional[ThresholdGuardrail] = None
    violence_graphic: typing.Optional[ThresholdGuardrail] = None
    harassment: typing.Optional[ThresholdGuardrail] = None
    harassment_threatening: typing.Optional[ThresholdGuardrail] = None
    hate: typing.Optional[ThresholdGuardrail] = None
    hate_threatening: typing.Optional[ThresholdGuardrail] = None
    self_harm_instructions: typing.Optional[ThresholdGuardrail] = None
    self_harm: typing.Optional[ThresholdGuardrail] = None
    self_harm_intent: typing.Optional[ThresholdGuardrail] = None
    sexual_minors: typing.Optional[ThresholdGuardrail] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.audio_output_multi import AudioOutputMulti
from ...types.final_output_multi import FinalOutputMulti

ReceiveMessageMulti = typing.Union[AudioOutputMulti, FinalOutputMulti]



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.voice_sample_preview_response_model import VoiceSamplePreviewResponseModel
from .raw_client import AsyncRawAudioClient, RawAudioClient


class AudioClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAudioClient
        """
        return self._raw_client

    def get(
        self,
        voice_id: str,
        sample_id: str,
        *,
        remove_background_noise: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VoiceSamplePreviewResponseModel:
        """
        Retrieve the first 30 seconds of voice sample audio with or without noise removal.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        sample_id : str
            Sample ID to be used

        remove_background_noise : typing.Optional[bool]
            If set will remove background noise for voice samples using our audio isolation model. If the samples do not include background noise, it can make the quality worse.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoiceSamplePreviewResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.samples.audio.get(
            voice_id="21m00Tcm4TlvDq8ikWAM",
            sample_id="VW7YKqPnjY4h39yTbx2L",
            remove_background_noise=True,
        )
        """
        _response = self._raw_client.get(
            voice_id, sample_id, remove_background_noise=remove_background_noise, request_options=request_options
        )
        return _response.data


class AsyncAudioClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAudioClient
        """
        return self._raw_client

    async def get(
        self,
        voice_id: str,
        sample_id: str,
        *,
        remove_background_noise: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VoiceSamplePreviewResponseModel:
        """
        Retrieve the first 30 seconds of voice sample audio with or without noise removal.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        sample_id : str
            Sample ID to be used

        remove_background_noise : typing.Optional[bool]
            If set will remove background noise for voice samples using our audio isolation model. If the samples do not include background noise, it can make the quality worse.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VoiceSamplePreviewResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.samples.audio.get(
                voice_id="21m00Tcm4TlvDq8ikWAM",
                sample_id="VW7YKqPnjY4h39yTbx2L",
                remove_background_noise=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            voice_id, sample_id, remove_background_noise=remove_background_noise, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .conversation_source import ConversationSource
from .manual_source import ManualSource

MemoryEntrySearchResultSource = typing.Union[ConversationSource, ManualSource]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.create_agent_test_response_model import CreateAgentTestResponseModel
from ...types.get_tests_page_response_model import GetTestsPageResponseModel
from ...types.get_tests_summaries_by_ids_response_model import GetTestsSummariesByIdsResponseModel
from ...types.test_sharing_mode import TestSharingMode
from ...types.test_type import TestType
from .types.tests_create_request_body import TestsCreateRequestBody
from .types.tests_get_response import TestsGetResponse
from .types.tests_list_request_sort_mode import TestsListRequestSortMode
from .types.tests_update_request_body import TestsUpdateRequestBody
from .types.tests_update_response import TestsUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: TestsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateAgentTestResponseModel]:
        """
        Creates a new agent response test.

        Parameters
        ----------
        request : TestsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateAgentTestResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/create",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=TestsCreateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAgentTestResponseModel,
                    construct_type(
                        type_=CreateAgentTestResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def move(
        self,
        *,
        entity_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Any]:
        """
        Moves multiple tests or folders from one folder to another.

        Parameters
        ----------
        entity_ids : typing.Sequence[str]
            The IDs of tests or folders to move.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Tests or folders successfully moved to another folder
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/bulk-move",
            method="POST",
            json={
                "entity_ids": entity_ids,
                "move_to": move_to,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TestsGetResponse]:
        """
        Gets an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TestsGetResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestsGetResponse,
                    construct_type(
                        type_=TestsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, test_id: str, *, request: TestsUpdateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TestsUpdateResponse]:
        """
        Updates an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request : TestsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TestsUpdateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=TestsUpdateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestsUpdateResponse,
                    construct_type(
                        type_=TestsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Any]:
        """
        Deletes an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def summaries(
        self, *, test_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTestsSummariesByIdsResponseModel]:
        """
        Gets multiple agent response tests by their IDs. Returns a dictionary mapping test IDs to test summaries.

        Parameters
        ----------
        test_ids : typing.Sequence[str]
            List of test IDs to fetch. No duplicates allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTestsSummariesByIdsResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/summaries",
            method="POST",
            json={
                "test_ids": test_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTestsSummariesByIdsResponseModel,
                    construct_type(
                        type_=GetTestsSummariesByIdsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        parent_folder_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]] = None,
        include_folders: typing.Optional[bool] = None,
        sort_mode: typing.Optional[TestsListRequestSortMode] = None,
        sharing_mode: typing.Optional[TestSharingMode] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetTestsPageResponseModel]:
        """
        Lists all agent response tests with pagination support and optional search filtering.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many Tests to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search query to filter tests by name.

        parent_folder_id : typing.Optional[str]
            Filter by parent folder ID. Use 'root' to get items in the root folder.

        types : typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]]
            If present, the endpoint will return only tests/folders of the given types.

        include_folders : typing.Optional[bool]
            Deprecated. Use the `types` query param and include `folder` instead.

        sort_mode : typing.Optional[TestsListRequestSortMode]
            Sort mode for listing tests. Use 'folders_first' to place folders before tests.

        sharing_mode : typing.Optional[TestSharingMode]
            Filter test visibility. Use `shared_with_me` to return only tests/folders shared with the current user that they did not create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTestsPageResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing",
            method="GET",
            params={
                "cursor": cursor,
                "page_size": page_size,
                "search": search,
                "parent_folder_id": parent_folder_id,
                "types": types,
                "include_folders": include_folders,
                "sort_mode": sort_mode,
                "sharing_mode": sharing_mode,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTestsPageResponseModel,
                    construct_type(
                        type_=GetTestsPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: TestsCreateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateAgentTestResponseModel]:
        """
        Creates a new agent response test.

        Parameters
        ----------
        request : TestsCreateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateAgentTestResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/create",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=TestsCreateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAgentTestResponseModel,
                    construct_type(
                        type_=CreateAgentTestResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def move(
        self,
        *,
        entity_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Moves multiple tests or folders from one folder to another.

        Parameters
        ----------
        entity_ids : typing.Sequence[str]
            The IDs of tests or folders to move.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Tests or folders successfully moved to another folder
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/bulk-move",
            method="POST",
            json={
                "entity_ids": entity_ids,
                "move_to": move_to,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TestsGetResponse]:
        """
        Gets an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TestsGetResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestsGetResponse,
                    construct_type(
                        type_=TestsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, test_id: str, *, request: TestsUpdateRequestBody, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TestsUpdateResponse]:
        """
        Updates an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request : TestsUpdateRequestBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TestsUpdateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=TestsUpdateRequestBody, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestsUpdateResponse,
                    construct_type(
                        type_=TestsUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, test_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Deletes an agent response test by ID.

        Parameters
        ----------
        test_id : str
            The id of a chat response test. This is returned on test creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agent-testing/{jsonable_encoder(test_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def summaries(
        self, *, test_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTestsSummariesByIdsResponseModel]:
        """
        Gets multiple agent response tests by their IDs. Returns a dictionary mapping test IDs to test summaries.

        Parameters
        ----------
        test_ids : typing.Sequence[str]
            List of test IDs to fetch. No duplicates allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTestsSummariesByIdsResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing/summaries",
            method="POST",
            json={
                "test_ids": test_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTestsSummariesByIdsResponseModel,
                    construct_type(
                        type_=GetTestsSummariesByIdsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        parent_folder_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]] = None,
        include_folders: typing.Optional[bool] = None,
        sort_mode: typing.Optional[TestsListRequestSortMode] = None,
        sharing_mode: typing.Optional[TestSharingMode] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetTestsPageResponseModel]:
        """
        Lists all agent response tests with pagination support and optional search filtering.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many Tests to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            Search query to filter tests by name.

        parent_folder_id : typing.Optional[str]
            Filter by parent folder ID. Use 'root' to get items in the root folder.

        types : typing.Optional[typing.Union[TestType, typing.Sequence[TestType]]]
            If present, the endpoint will return only tests/folders of the given types.

        include_folders : typing.Optional[bool]
            Deprecated. Use the `types` query param and include `folder` instead.

        sort_mode : typing.Optional[TestsListRequestSortMode]
            Sort mode for listing tests. Use 'folders_first' to place folders before tests.

        sharing_mode : typing.Optional[TestSharingMode]
            Filter test visibility. Use `shared_with_me` to return only tests/folders shared with the current user that they did not create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTestsPageResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/agent-testing",
            method="GET",
            params={
                "cursor": cursor,
                "page_size": page_size,
                "search": search,
                "parent_folder_id": parent_folder_id,
                "types": types,
                "include_folders": include_folders,
                "sort_mode": sort_mode,
                "sharing_mode": sharing_mode,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTestsPageResponseModel,
                    construct_type(
                        type_=GetTestsPageResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.allowed_output_formats import AllowedOutputFormats
from .raw_client import AsyncRawTextToSoundEffectsClient, RawTextToSoundEffectsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TextToSoundEffectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTextToSoundEffectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTextToSoundEffectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTextToSoundEffectsClient
        """
        return self._raw_client

    def convert(
        self,
        *,
        text: str,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        loop: typing.Optional[bool] = OMIT,
        duration_seconds: typing.Optional[float] = OMIT,
        prompt_influence: typing.Optional[float] = OMIT,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Turn text into sound effects for your videos, voice-overs or video games using the most advanced sound effects models in the world.

        Parameters
        ----------
        text : str
            The text that will get converted into a sound effect.

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        loop : typing.Optional[bool]
            Whether to create a sound effect that loops smoothly. Only available for the 'eleven_text_to_sound_v2 model'.

        duration_seconds : typing.Optional[float]
            The duration of the sound which will be generated in seconds. Must be at least 0.5 and at most 30. If set to None we will guess the optimal duration using the prompt. Defaults to None.

        prompt_influence : typing.Optional[float]
            A higher prompt influence makes your generation follow the prompt more closely while also making generations less variable. Must be a value between 0 and 1. Defaults to 0.3.

        model_id : typing.Optional[str]
            The model ID to use for the sound generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            The generated sound effect as an MP3 file

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.text_to_sound_effects.convert(
            text="Spacious braam suitable for high-impact movie trailer moments",
        )
        """
        with self._raw_client.convert(
            text=text,
            output_format=output_format,
            loop=loop,
            duration_seconds=duration_seconds,
            prompt_influence=prompt_influence,
            model_id=model_id,
            request_options=request_options,
        ) as r:
            yield from r.data


class AsyncTextToSoundEffectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTextToSoundEffectsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTextToSoundEffectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTextToSoundEffectsClient
        """
        return self._raw_client

    async def convert(
        self,
        *,
        text: str,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        loop: typing.Optional[bool] = OMIT,
        duration_seconds: typing.Optional[float] = OMIT,
        prompt_influence: typing.Optional[float] = OMIT,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Turn text into sound effects for your videos, voice-overs or video games using the most advanced sound effects models in the world.

        Parameters
        ----------
        text : str
            The text that will get converted into a sound effect.

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        loop : typing.Optional[bool]
            Whether to create a sound effect that loops smoothly. Only available for the 'eleven_text_to_sound_v2 model'.

        duration_seconds : typing.Optional[float]
            The duration of the sound which will be generated in seconds. Must be at least 0.5 and at most 30. If set to None we will guess the optimal duration using the prompt. Defaults to None.

        prompt_influence : typing.Optional[float]
            A higher prompt influence makes your generation follow the prompt more closely while also making generations less variable. Must be a value between 0 and 1. Defaults to 0.3.

        model_id : typing.Optional[str]
            The model ID to use for the sound generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            The generated sound effect as an MP3 file

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.text_to_sound_effects.convert(
                text="Spacious braam suitable for high-impact movie trailer moments",
            )


        asyncio.run(main())
        """
        async with self._raw_client.convert(
            text=text,
            output_format=output_format,
            loop=loop,
            duration_seconds=duration_seconds,
            prompt_influence=prompt_influence,
            model_id=model_id,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

BackgroundSoundPresetId = typing.Union[
    typing.Literal[
        "office2", "office1", "restaurant", "city", "typing", "elevator1", "elevator2", "elevator3", "elevator4"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .array_json_schema_property_output_constant_value_item import ArrayJsonSchemaPropertyOutputConstantValueItem


class ArrayJsonSchemaPropertyOutput(UncheckedBaseModel):
    type: typing.Optional[typing.Literal["array"]] = None
    description: typing.Optional[str] = None
    items: typing.Optional["ArrayJsonSchemaPropertyOutputItems"] = pydantic.Field(default=None)
    """
    Schema for array elements.
    """

    dynamic_variable: typing.Optional[str] = pydantic.Field(default=None)
    """
    When set, the entire array is populated from this dynamic variable at runtime. Mutually exclusive with description (LLM-provided array), constant_value, and is_omitted.
    """

    constant_value: typing.Optional[typing.List[ArrayJsonSchemaPropertyOutputConstantValueItem]] = pydantic.Field(
        default=None
    )
    """
    When set, the entire array uses this constant value at runtime. Mutually exclusive with description (LLM-provided array), dynamic_variable, and is_omitted.
    """

    is_omitted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, this array parameter will be completely omitted from the request. Only valid for optional parameters. Mutually exclusive with description, dynamic_variable, and constant_value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .object_json_schema_property_output import ObjectJsonSchemaPropertyOutput  # noqa: E402, I001
from .array_json_schema_property_output_items import ArrayJsonSchemaPropertyOutputItems  # noqa: E402, I001

update_forward_refs(ArrayJsonSchemaPropertyOutput, ObjectJsonSchemaPropertyOutput=ObjectJsonSchemaPropertyOutput)



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceResponseModelPaymentIntentStatus = typing.Union[
    typing.Literal[
        "canceled",
        "processing",
        "requires_action",
        "requires_capture",
        "requires_confirmation",
        "requires_payment_method",
        "succeeded",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .tool_request_model_tool_config import ToolRequestModelToolConfig
from .tool_response_mock_config_input import ToolResponseMockConfigInput


class ToolRequestModel(UncheckedBaseModel):
    tool_config: ToolRequestModelToolConfig = pydantic.Field()
    """
    Configuration for the tool
    """

    response_mocks: typing.Optional[typing.List[ToolResponseMockConfigInput]] = pydantic.Field(default=None)
    """
    Mock responses with optional parameter conditions. Evaluated top-to-bottom; first match wins.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(ToolRequestModel)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class ProjectVoiceResponseModel(UncheckedBaseModel):
    project_voice_ref_id: str
    voice_id: str
    alias: str
    stability: float
    similarity_boost: float
    style: float
    is_pinned: bool
    use_speaker_boost: bool
    volume_gain: float
    speed: float

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.segment_create_response import SegmentCreateResponse
from .raw_client import AsyncRawSegmentClient, RawSegmentClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SegmentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSegmentClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSegmentClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSegmentClient
        """
        return self._raw_client

    def create(
        self,
        dubbing_id: str,
        speaker_id: str,
        *,
        start_time: float,
        end_time: float,
        text: typing.Optional[str] = OMIT,
        translations: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentCreateResponse:
        """
        Creates a new segment in dubbing resource with a start and end time for the speaker in every available language. Does not automatically generate transcripts/translations/audio.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        speaker_id : str
            ID of the speaker.

        start_time : float

        end_time : float

        text : typing.Optional[str]

        translations : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentCreateResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.resource.speaker.segment.create(
            dubbing_id="dubbing_id",
            speaker_id="speaker_id",
            start_time=1.1,
            end_time=1.1,
        )
        """
        _response = self._raw_client.create(
            dubbing_id,
            speaker_id,
            start_time=start_time,
            end_time=end_time,
            text=text,
            translations=translations,
            request_options=request_options,
        )
        return _response.data


class AsyncSegmentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSegmentClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSegmentClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSegmentClient
        """
        return self._raw_client

    async def create(
        self,
        dubbing_id: str,
        speaker_id: str,
        *,
        start_time: float,
        end_time: float,
        text: typing.Optional[str] = OMIT,
        translations: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentCreateResponse:
        """
        Creates a new segment in dubbing resource with a start and end time for the speaker in every available language. Does not automatically generate transcripts/translations/audio.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        speaker_id : str
            ID of the speaker.

        start_time : float

        end_time : float

        text : typing.Optional[str]

        translations : typing.Optional[typing.Dict[str, typing.Optional[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentCreateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.resource.speaker.segment.create(
                dubbing_id="dubbing_id",
                speaker_id="speaker_id",
                start_time=1.1,
                end_time=1.1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            dubbing_id,
            speaker_id,
            start_time=start_time,
            end_time=end_time,
            text=text,
            translations=translations,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.column_filter import ColumnFilter
from ....types.workspace_analytics_query_response_model import WorkspaceAnalyticsQueryResponseModel
from .raw_client import AsyncRawRequestsClient, RawRequestsClient
from .types.body_list_api_requests_v_1_workspace_analytics_requests_post_sort import (
    BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RequestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRequestsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRequestsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRequestsClient
        """
        return self._raw_client

    def get(
        self,
        *,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort: typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort] = OMIT,
        filters: typing.Optional[typing.Sequence[ColumnFilter]] = OMIT,
        search: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceAnalyticsQueryResponseModel:
        """
        Returns a list of API requests. Supports filtering by time range, column filters, and search terms. At least one of start_time or end_time must be provided. An optional sort parameter controls timestamp ordering. Results are ordered by timestamp. Descending if end_time is used, ascending if start_time is used. The response is a tabular structure with columns, column_types, column_units, and rows.

        Parameters
        ----------
        start_time : typing.Optional[int]
            Start of the time range as a Unix timestamp in milliseconds.

        end_time : typing.Optional[int]
            End of the time range as a Unix timestamp in milliseconds.

        limit : typing.Optional[int]

        sort : typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort]
            Optional timestamp sort direction. If omitted, defaults to desc when end_time is provided, otherwise asc.

        filters : typing.Optional[typing.Sequence[ColumnFilter]]

        search : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceAnalyticsQueryResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.analytics.requests.get()
        """
        _response = self._raw_client.get(
            start_time=start_time,
            end_time=end_time,
            limit=limit,
            sort=sort,
            filters=filters,
            search=search,
            request_options=request_options,
        )
        return _response.data


class AsyncRequestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRequestsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRequestsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRequestsClient
        """
        return self._raw_client

    async def get(
        self,
        *,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort: typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort] = OMIT,
        filters: typing.Optional[typing.Sequence[ColumnFilter]] = OMIT,
        search: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceAnalyticsQueryResponseModel:
        """
        Returns a list of API requests. Supports filtering by time range, column filters, and search terms. At least one of start_time or end_time must be provided. An optional sort parameter controls timestamp ordering. Results are ordered by timestamp. Descending if end_time is used, ascending if start_time is used. The response is a tabular structure with columns, column_types, column_units, and rows.

        Parameters
        ----------
        start_time : typing.Optional[int]
            Start of the time range as a Unix timestamp in milliseconds.

        end_time : typing.Optional[int]
            End of the time range as a Unix timestamp in milliseconds.

        limit : typing.Optional[int]

        sort : typing.Optional[BodyListApiRequestsV1WorkspaceAnalyticsRequestsPostSort]
            Optional timestamp sort direction. If omitted, defaults to desc when end_time is provided, otherwise asc.

        filters : typing.Optional[typing.Sequence[ColumnFilter]]

        search : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceAnalyticsQueryResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.analytics.requests.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            start_time=start_time,
            end_time=end_time,
            limit=limit,
            sort=sort,
            filters=filters,
            search=search,
            request_options=request_options,
        )
        return _response.data



"""Types for the Speech Engine module."""

import typing

import pydantic

# ---------------------------------------------------------------------------
# Event name constants
# ---------------------------------------------------------------------------

INIT = "init"
USER_TRANSCRIPT = "user_transcript"
CLOSE = "close"
ERROR = "error"
DISCONNECTED = "disconnected"

# ---------------------------------------------------------------------------
# Wire protocol — incoming (ElevenLabs API -> developer server)
# ---------------------------------------------------------------------------
#
# InitMessage:           {"type": "init", "conversation_id": "..."}
# UserTranscriptMessage: {"type": "user_transcript", "user_transcript": [...], "event_id": N}
# PingMessage:           {"type": "ping"}
# CloseMessage:          {"type": "close"}
# ErrorMessage:          {"type": "error", "message": "..."}
#
# ---------------------------------------------------------------------------
# Wire protocol — outgoing (developer server -> ElevenLabs API)
# ---------------------------------------------------------------------------
#
# AgentResponseMessage:  {"type": "agent_response", "content": "...", "event_id": N, "is_final": bool}
# PongMessage:           {"type": "pong"}
#

# ---------------------------------------------------------------------------
# ConversationMessage
# ---------------------------------------------------------------------------


class ConversationMessage(pydantic.BaseModel):
    """A single message in a speech engine conversation.

    Attributes:
        role: Either ``"user"`` or ``"agent"``.
        content: The text content of the message.
    """

    role: str
    content: str


# ---------------------------------------------------------------------------
# WebSocket abstraction
# ---------------------------------------------------------------------------


class WebSocketLike(typing.Protocol):
    """Protocol for WebSocket-like objects.

    Compatible with ``websockets.WebSocketServerProtocol`` and
    FastAPI/Starlette ``WebSocket`` out of the box (auto-detected).
    """

    async def recv(self) -> typing.Union[str, bytes]:
        ...  # pragma: no cover

    async def send(self, data: str) -> None:
        ...  # pragma: no cover

    async def close(self) -> None:
        ...  # pragma: no cover


class _ASGIWebSocketAdapter:
    """Adapts a FastAPI/Starlette WebSocket to the :class:`WebSocketLike`
    interface expected by :class:`~.session.SpeechEngineSession`."""

    def __init__(self, ws: typing.Any) -> None:
        self._ws = ws

    async def recv(self) -> typing.Union[str, bytes]:
        return await self._ws.receive_text()

    async def send(self, data: str) -> None:
        await self._ws.send_text(data)

    async def close(self) -> None:
        await self._ws.close()


def wrap_websocket(ws: typing.Any) -> WebSocketLike:
    """Return a :class:`WebSocketLike` wrapper for *ws*.

    If *ws* already has ``recv``/``send`` (e.g. the ``websockets`` library),
    it is returned as-is.  If it has ``receive_text``/``send_text`` (e.g.
    FastAPI/Starlette), it is wrapped with :class:`_ASGIWebSocketAdapter`.
    """
    if hasattr(ws, "recv"):
        return ws
    if hasattr(ws, "receive_text"):
        return _ASGIWebSocketAdapter(ws)
    raise TypeError(
        f"Cannot wrap {type(ws).__name__}: expected a websockets-style "
        f"object (recv/send) or an ASGI-style object (receive_text/send_text)"
    )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .utterance_response_model import UtteranceResponseModel


class SpeakerResponseModel(UncheckedBaseModel):
    speaker_id: str = pydantic.Field()
    """
    The ID of the speaker.
    """

    duration_secs: float = pydantic.Field()
    """
    The duration of the speaker segment in seconds.
    """

    utterances: typing.Optional[typing.List[UtteranceResponseModel]] = pydantic.Field(default=None)
    """
    The utterances of the speaker.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

RagIndexStatus = typing.Union[
    typing.Literal[
        "new",
        "created",
        "processing",
        "failed",
        "succeeded",
        "rag_limit_exceeded",
        "document_too_small",
        "cannot_index_folder",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .caption_style_model import CaptionStyleModel
from .project_creation_meta_response_model import ProjectCreationMetaResponseModel
from .project_response_model_access_level import ProjectResponseModelAccessLevel
from .project_response_model_aspect_ratio import ProjectResponseModelAspectRatio
from .project_response_model_fiction import ProjectResponseModelFiction
from .project_response_model_source_type import ProjectResponseModelSourceType
from .project_response_model_target_audience import ProjectResponseModelTargetAudience
from .project_state import ProjectState
from .studio_agent_settings_model import StudioAgentSettingsModel


class ProjectResponse(UncheckedBaseModel):
    project_id: str = pydantic.Field()
    """
    The ID of the project.
    """

    name: str = pydantic.Field()
    """
    The name of the project.
    """

    create_date_unix: int = pydantic.Field()
    """
    The creation date of the project.
    """

    created_by_user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user ID who created the project.
    """

    default_title_voice_ref_id: str = pydantic.Field()
    """
    The default title project voice reference ID.
    """

    default_paragraph_voice_ref_id: str = pydantic.Field()
    """
    The default paragraph project voice reference ID.
    """

    default_model_id: str = pydantic.Field()
    """
    The default model ID.
    """

    last_conversion_date_unix: typing.Optional[int] = pydantic.Field(default=None)
    """
    The last conversion date of the project.
    """

    can_be_downloaded: bool = pydantic.Field()
    """
    Whether the project can be downloaded.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the project.
    """

    author: typing.Optional[str] = pydantic.Field(default=None)
    """
    The author of the project.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the project.
    """

    genres: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of genres of the project.
    """

    cover_image_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cover image URL of the project.
    """

    target_audience: typing.Optional[ProjectResponseModelTargetAudience] = pydantic.Field(default=None)
    """
    The target audience of the project.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    Two-letter language code (ISO 639-1) of the language of the project.
    """

    content_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The content type of the project, e.g. 'Novel' or 'Short Story'
    """

    original_publication_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The original publication date of the project.
    """

    mature_content: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the project contains mature content.
    """

    isbn_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ISBN number of the project.
    """

    volume_normalization: bool = pydantic.Field()
    """
    Whether the project uses volume normalization.
    """

    state: ProjectState = pydantic.Field()
    """
    The state of the project.
    """

    access_level: ProjectResponseModelAccessLevel = pydantic.Field()
    """
    The access level of the project.
    """

    fiction: typing.Optional[ProjectResponseModelFiction] = pydantic.Field(default=None)
    """
    Whether the project is fiction.
    """

    quality_check_on: bool = pydantic.Field()
    """
    Whether quality check is enabled for this project.
    """

    quality_check_on_when_bulk_convert: bool = pydantic.Field()
    """
    Whether quality check is enabled on the project when bulk converting.
    """

    creation_meta: typing.Optional[ProjectCreationMetaResponseModel] = pydantic.Field(default=None)
    """
    The creation meta of the project.
    """

    source_type: typing.Optional[ProjectResponseModelSourceType] = pydantic.Field(default=None)
    """
    The source type of the project.
    """

    chapters_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether chapters are enabled for the project.
    """

    captions_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether captions are enabled for the project.
    """

    caption_style: typing.Optional[CaptionStyleModel] = pydantic.Field(default=None)
    """
    Global styling to be applied to all captions
    """

    caption_style_template_overrides: typing.Optional[typing.Dict[str, typing.Optional[CaptionStyleModel]]] = (
        pydantic.Field(default=None)
    )
    """
    Styling changes that have been made to the provided templates
    """

    public_share_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The public share ID of the project.
    """

    aspect_ratio: typing.Optional[ProjectResponseModelAspectRatio] = pydantic.Field(default=None)
    """
    The aspect ratio of the project.
    """

    agent_settings: typing.Optional[StudioAgentSettingsModel] = pydantic.Field(default=None)
    """
    Agent-related settings for the project
    """

    default_title_voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The default title voice ID.
    """

    default_paragraph_voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The default paragraph voice ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_config_override_config import AgentConfigOverrideConfig
from .asr_conversational_config_override_config import AsrConversationalConfigOverrideConfig
from .conversation_config_override_config import ConversationConfigOverrideConfig
from .tts_conversational_config_override_config import TtsConversationalConfigOverrideConfig
from .turn_config_override_config import TurnConfigOverrideConfig


class ConversationConfigClientOverrideConfigInput(UncheckedBaseModel):
    asr: typing.Optional[AsrConversationalConfigOverrideConfig] = pydantic.Field(default=None)
    """
    Configures overrides for nested fields.
    """

    turn: typing.Optional[TurnConfigOverrideConfig] = pydantic.Field(default=None)
    """
    Configures overrides for nested fields.
    """

    tts: typing.Optional[TtsConversationalConfigOverrideConfig] = pydantic.Field(default=None)
    """
    Configures overrides for nested fields.
    """

    conversation: typing.Optional[ConversationConfigOverrideConfig] = pydantic.Field(default=None)
    """
    Configures overrides for nested fields.
    """

    agent: typing.Optional[AgentConfigOverrideConfig] = pydantic.Field(default=None)
    """
    Configures overrides for nested fields.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .llm_literal_json_schema_property import LlmLiteralJsonSchemaProperty


class AstllmNodeOutput(UncheckedBaseModel):
    value_schema: LlmLiteralJsonSchemaProperty = pydantic.Field()
    """
    JSON schema describing the value that the LLM should extract.
    """

    prompt: str = pydantic.Field()
    """
    The prompt to evaluate to a boolean value. Deprecated. Use a boolean schema instead.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .column_unit import ColumnUnit
from .workspace_analytics_query_response_model_column_types_item import (
    WorkspaceAnalyticsQueryResponseModelColumnTypesItem,
)
from .workspace_analytics_query_response_model_rows_item_item import WorkspaceAnalyticsQueryResponseModelRowsItemItem


class WorkspaceAnalyticsQueryResponseModel(UncheckedBaseModel):
    columns: typing.List[str]
    column_types: typing.List[WorkspaceAnalyticsQueryResponseModelColumnTypesItem]
    rows: typing.List[typing.List[typing.Optional[WorkspaceAnalyticsQueryResponseModelRowsItemItem]]]
    column_units: typing.List[typing.Optional[ColumnUnit]]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DirectPublishingReadResponseModelPayoutType = typing.Union[
    typing.Literal["none", "engagement_based", "fixed_payout"], typing.Any
]
